require 'jsduck/batch_parser'
require 'jsduck/batch_processor'
require 'jsduck/assets'
require 'jsduck/tag_registry'
require 'jsduck/export_writer'
require 'jsduck/web_writer'

module JsDuck

  # The main application logic of jsduck
  class App
    # Initializes app with JsDuck::Options object
    def initialize(opts)
      @opts = opts
    end

    # Main App logic.
    def run
      parse

      init_assets

      if @opts.export
        generate_export
      else
        generate_web_page
      end
    end

    private

    def parse
      @parsed_files = BatchParser.parse(@opts)
      @relations = BatchProcessor.process(@parsed_files, @opts)
    end

    def init_assets
      # Initialize guides, videos, examples, ...
      @assets = Assets.new(@relations, @opts)

      # HACK: Give access to assets from @aside tag
      TagRegistry.get_by_key(:aside).assets = @assets
    end

    def generate_export
      ExportWriter.new(@relations, @assets, @opts).write
    end

    def generate_web_page
      WebWriter.new(@relations, @assets, @parsed_files, @opts).write
    end

  end

end
