require "jsduck/tag/tag"

module JsDuck::Tag
  # There is no @name tag.
  #
  # The :name field is generated by several tags like @class, @method,
  # @cfg, ...
  #
  # This class exists to take care of the merging of :name field.
  class Name < Tag
    def initialize
      @merge_context = [:class, :member]
    end

    # When docs contains :name, it's taken from there.
    #
    # When code contains :name, we only take the full name when
    # dealing with a class, otherwise we just take the last part of
    # the dot.separate.name.
    #
    # :name defaults to empty string.
    def merge(h, docs, code)
      if docs[:name]
        h[:name] = docs[:name]
      elsif code[:name]
        if h[:tagname] == :class
          h[:name] = code[:name]
        else
          h[:name] = code[:name].split(/\./).last
        end
      else
        h[:name] = ""
      end
    end
  end
end
